#include <stdio.h>
#include <stdlib.h>
#include <glob.h>
#include <dlfcn.h>

#include "binding_load.h"
#include "papyrus_data.h"
#include "mem.h"

#define NBINDINGPATHS 4
char *binding_search_path[NBINDINGPATHS]={"./", "./.libs/", "/usr/lib/papyrus/", 
														"/usr/local/lib/papyrus/"};

static void *check_shared_library (char *so)
	{
	void *handle;
	void *sym;
	
	if ((handle = dlopen (so, RTLD_LAZY)) == 0)
		return NULL;
	
	if ((sym = dlsym (handle, "papyrus_binding_init")) == 0)
		{
		dlclose (handle);
		return NULL;
		}
		
	return handle;
	}

papyrus_database_info *binding_init (char *binding, char *args)
	{
	papyrus_database_info *db_info = NULL;
	int i, count;
	glob_t g;
	char *pattern;
	
	db_info = mem_alloc (sizeof (*db_info));
	db_info->binding = binding ? mem_strdup (binding) : NULL;
	db_info->args = args ? mem_strdup (args) : NULL;
	
	if (!db_info)
		return NULL;

	for (i = 0; i < NBINDINGPATHS; i++)
		{
		asprintf (&pattern, "%s/*.so", binding_search_path [i]);
		
		if (glob (pattern, 0, NULL, &g) == 0)
			{
			for (count = 0; count < g.gl_pathc; count++)
				{
				msg_debug ("Checking library: %s", g.gl_pathv[count]);
				
				db_info->dlhandle = check_shared_library (g.gl_pathv[count]);
				
				if (db_info->dlhandle)
					{
					db_info->sym = dlsym (db_info->dlhandle, "papyrus_binding_init");
					g_assert (db_info->sym);
					if (db_info->sym (db_info) == 0)
						{
						globfree (&g);
						return db_info;
						}
					}
					
				}
			}
		
		free (pattern);
		}
	
	mem_free (db_info->binding);
	mem_free (db_info->args);
	mem_free (db_info);
		
	return NULL;
	}

