#include <stdio.h>
#include <stdlib.h>

#include <glib.h>

#include "xml_layout.h"
#include "layout_render.h"
#include "mem.h"
#include "papyrus_data.h"


static void	build_item_width_height (render_item *render)
	{
	if (!render)
		return;
	
	render->width = 0;
	render->height = 0;
	
	if (!render->layout)
		{
		msg_error ("Render object has no associated item!");
		return;
		}
	
	switch (render->type)
		{
		case RENDER_string :
			{
			render->width = gnome_font_get_width_utf8 (render->layout->font, render->data);
			render->height = 
				gnome_font_get_ascender (render->layout->font) +
				gnome_font_get_descender (render->layout->font) +
				gnome_font_get_underline_thickness (render->layout->font);
			break;
			}
			
		default :
			msg_error ("Invalid render type: %d", render->type);
		}
	}

static render_item *layout_add_string (layout_item *item)
	{
	render_item *render;
	
	msg_debug ("got item: %s (%s)", item->name, item->data);
	
	render = (render_item*) mem_allocz (sizeof (render_item));
	render->type = RENDER_string;
	render->data = mem_strdup (item->data);
	
	render->layout = item;
	render->parent = item->parent ? item->parent->render : NULL;
	
	build_item_width_height (render);
	
	if (item->parent)
		{
		render->parent = item->render;
		render->rel_y = 0;
		render->rel_x = 0;
		
		switch (item->parent->pos)
			{
			case LAYOUT_left : render->rel_x = -render->width; break;
			case LAYOUT_right : render->rel_x = render->parent->width; break;
			case LAYOUT_above : render->rel_y = -render->height; break;
			case LAYOUT_below : render->rel_y = render->parent->height; break;
			default :
				msg_debug ("blah! unknown position type: %d", item->parent->pos);
			}
		}
	else
		{
		msg_error ("Non-parented item, don't know what to do");
		render->rel_y = 0; render->rel_x = 0;
		}
		
	return render;
	}

static GList *layout_add_group (layout_item *group)
	{
	GList *cur, *cur2, *retval = NULL;
	layout_item *item;
	render_item *render;
	
	if (!group)
		return NULL;
	
	for (cur = group->children; cur != NULL; cur = cur->next)
		{
		item = cur->data;
		
		if (!item)
			continue;
		
		render = NULL;
		
		switch (item->type)
			{
			case LAYOUT_string :
				{
				render = layout_add_string (item);
				break;
				}
				
			default:
				msg_error ("Invalid layout item: %d", item->type);
			}
		
		if (render)		
			retval = g_list_append (retval, render);
		
		if (item->children)
			for (cur2 = item->children; cur2 != NULL; cur2 = cur2->next)
				retval = g_list_append (retval, layout_add_group (cur2->data));
		}
		
	return retval;
	}

GList *layout_to_render (papyrus_database_info *db_info, GList *layouts)
	{
	GList *retval = NULL;
	GList *cur;
	report_item *item;
	
	for (cur = layouts; cur != NULL; cur = cur->next)
		{
		item = cur->data;
		switch (item->type)
			{
			case REPORT_group:
				{
				retval = g_list_concat (retval, layout_add_group (item->body));
				break;
				}
				
			default:
				msg_error ("Invalid report item type: %d", item->type);
			}
		}
	
	return retval;
	}
