#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include "mem.h"
#include "xml2print.h"

typedef struct
{
	char *name;
	char *value;
} arg_pair;

static arg_pair *parse_arguments (int argc, char **args)
{
	int i, pos;
	arg_pair *retval;

	retval = mem_alloc (sizeof (arg_pair) * argc);

	for (i = 0; i < argc; i++)
	{
		pos = (strstr (args[i], "=") - args[i]);
		if (!pos)
		{
			msg_error ("Invalid argument: %s\n", args[i]);
			continue;
		}
	
		retval[i].name = mem_strndup (args[i], pos);
		retval[i].name[pos] = '\0';
		retval[i].value = mem_strdup (&args[i][pos + 1]);
	}
	
	return retval;
}

static void display_xml_tree_node (xmlNode *node, int depth);

static void display_xml_tree_attr (xmlAttr *attr, int depth)
{
	if (!attr)
		return;
	msg_display ("%*sattr: %s", depth * 2, "", attr->name);
#if 0 /* this is the correct one */	
	display_xml_tree_node (attr->val, depth + 1);
#else /* but this one looks nice */
	msg_display ("%*s%s: %s", depth * 2, "", attr->val->name, attr->val->content);
#endif
	display_xml_tree_attr (attr->next, depth);
}

static void display_xml_tree_node (xmlNode *node, int depth)
{
	if (!node)
		return;
	msg_display ("%*snode: %s", depth * 2, "", node->name);
	msg_display ("%*scontent:\n=====\n%s\n=====", depth * 2, "", node->content);
	display_xml_tree_attr (node->properties, depth + 1);
	display_xml_tree_node (node->childs, depth + 1);
	display_xml_tree_node (node->next, depth);
}

static void display_xml_tree (xmlDoc *doc)
{
	msg_display ("tree: %s", doc->name);
	display_xml_tree_node (doc->root, 0);
}

int main (int argc, char *argv[])
{
	arg_pair *args;
	char *inputfile;
	xmlDocPtr doc;

	if (argc < 2)
	{
		msg_error ("%s: filename [ args ... ]\n", argv[0]);
		return 1;
	}

	inputfile = argv[1];
	
	args = parse_arguments (argc - 2, &argv[2]);
	
	doc = xmlParseFile (inputfile);
	
	msg_debug ("Not verifying document against dtd. fix");
	
	if (!doc)
	{
		msg_error ("Error parsing file: %s", inputfile);
		return 1;
	}
	
	display_xml_tree (doc);	
	
	xml_to_print (doc);
	
	return 0;
}
