#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>
#include <gnome.h>
#include <libgnomeprint/gnome-print.h>
#include <libgnomeprint/gnome-printer-dialog.h>
#include <libgnomeprint/gnome-print-master-preview.h>

#include "mem.h"
#include "print.h"
#include "xml2print.h"
#include "binding_load.h"
#include "xml_funcs.h"
#include "layout_render.h"
#include "xml_layout.h"

static char *get_paper_details (char *papername)
   {
   FILE *fp;
   int pos;
   char line[121];
   char tmppaper[50];
   static char details[121];

   if ((fp = fopen ("/etc/paper.config", "r")) == NULL)
      return NULL;

   while (fgets (line, 120, fp) != NULL)
      {
      if (strstr (line, "=") != NULL)
         {
         pos = strstr(line, "=") - line;
         strncpy (tmppaper, line, pos);
         tmppaper[pos] = '\0';
         }
      else
         continue;

      if (strcasecmp (papername, tmppaper) == 0)
         { 
         strcpy (details, &line[pos+2]);
         details[strlen(details)-2]='\0';
         return details;
         }
      }

   return NULL;
   }

static int print_paper_width (void)
   {
   char *details = get_paper_details (print_paper ());
	return details ? atoi(details) : 595; /* a4 */
   }

static int print_paper_height (void)
   {
   char *details = get_paper_details (print_paper ());
   return details ? atoi (&details[strstr(details, " ") - details]) : 841; /* a4 */
   }
	
static char *find_paper_size (xmlDoc *doc)
	{
	char *papersize = attr_has_property (doc->root->properties, "pagesize");
	return papersize ? papersize : "a4";
	}
	
static char *find_print_mode (xmlDoc *doc)
	{
	char *mode = attr_has_property (doc->root->properties, "mode");
	return mode ? mode : "preview";
	}
	
static char *find_binding (xmlDoc *doc)
	{
	return attr_has_property (doc->root->properties, "binding");
	}
	
static char *find_binding_arguments (xmlDoc *doc)
	{
	return attr_has_property (doc->root->properties, "arguments");
	}
		
static gboolean preview_close_check (gpointer data)
   {
#warning "data is a junk pointer, so this works. BUT IT IS SOOO BAD"
   if (!GTK_IS_WIDGET (data))
      {
		msg_error ("Discovered preview widget was closed. Quitting");
      gtk_main_quit ();
      return FALSE;
      }

   return TRUE;
   }

int xml_to_print (xmlDoc *doc)
	{
	char *paper = find_paper_size (doc);
	char *mode = find_print_mode (doc);
	char *binding = find_binding (doc);
	char *args = find_binding_arguments (doc);
	GList *layout, *render;
	papyrus_database_info *db_info;
	
	print_init (paper, mode);
	db_info = binding_init (binding, args);

	layout = xml_build_file (doc);
	render = layout_to_render (db_info, layout);
	
	if (strcasecmp (mode, "preview") == 0)
	{
		GnomePrintMasterPreview *preview;
	
		gnome_print_master_close (print_get_master ());
		preview = gnome_print_master_preview_new (print_get_master (), "Preview");
		gtk_widget_show_all (GTK_WIDGET (preview));
	
		g_timeout_add (400, preview_close_check, preview);
	
		gtk_main ();
	}
	
	gnome_print_context_close (print_get_context ());
	
	return 0;
	}
	